#!/bin/sh
#
# Script to install Seattle Firewall
#
#     This program is under GPL [http://www.gnu.org/copyleft/gpl.htm]         
#
#     (c) 2000 - Tom Eastep (teastep@evergo.net)
#
#       Seawall documentation is available at http://seawall.sourceforge.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of Version 2 of the GNU General Public License 
#       as published by the Free Software Foundation.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
#
#    Usage:
#
#        If you are running a distribution that has a directory called /etc/rc.d/init.d or one
#        called /etc/init.d or you are running Slackware then simply cd to the directory 
#        containing this script and run it.
#
#            ./install.sh
#
#        If you don't have either of those directories, you will need to determine where the
#        SysVInit scripts are kept on your system and pass the name of that directory.
#
#            ./install.sh /etc/rc.d/scripts
#
#        The default is that the firewall will be started in run levels 2-5 starting at
#        position 15 and stopping at position 90. This is correct RedHat/Mandrake, Debian, 
#        Caldera and Corel.
#
#        If you wish to change that, you can pass -r "<levels startpos stoppos>".
#
#        Example 1: You wish to start your firewall in runlevels 2 and three, start at position
#                   15 and stop at position 90
#
#            ./install.sh -r "23 15 90"
#
#       Example 2: You wish to start your firewall only in run level 3, start at position 5 
#                  and stop at position 95.
#
#            ./install.sh -r "3 5 95" /etc/rc.d/scripts
#
#        For distributions that don't include chkconfig (Slackware, for example), the
#        /etc/rc.d/rc.local file is modified to start the firewall.
#

VERSION=3.0

usage() # $1 = exit status
{
    ME=`basename $0`
    echo "usage: $ME [ -r \"<chkconfig parameters>\" ] [ <init scripts directory> ]"
    echo "       $ME [ -v ]"
    echo "       $ME [ -h ]"
    exit $1
}

run_install()
{
    if ( ! install $* ); then
	echo -e "\nERROR: Failed to install $*"
	exit 1
    fi
}

cant_autostart()
{
    echo -e "\nWARNING: Unable to configure Seattle Firewall to start"
echo    "           automatically at boot"
}

modify_rclocal()
{
    if [ -f /etc/rc.d/rc.local ]; then
	if [ -z "`grep seawall /etc/rc.d/rc.local`" ]; then
	    echo >> /etc/rc.d/rc.local
	    echo "/sbin/seawall start" >> /etc/rc.d/rc.local
	    echo "/etc/rc.d/rc.local modified to start Seattle Firewall"
	fi
    else
	cant_autostart
    fi	
}

#
# Parse the run line
#
# DEST is the SysVInit script directory
# RUNLEVELS is the chkconfig parmeters for firewall
# ARGS is "yes" if we've already parsed an argument
#
DEST=""
RUNLEVELS=""
ARGS=""

while [ $# -gt 0 ] ; do
    case "$1" in
	-h|help|?)
	    if [ -n "$ARGS" ]; then
		usage 1
            fi

	    usage 0
	    ;;
	-r)
	    if [ -n "$RUNLEVELS" -o $# -eq 1 ]; then
		usage 1
	    fi

	    RUNLEVELS="$2";
	    shift
	    ;;
        -v)
	    if [ -n "$ARGS" ]; then
		usage 1
            fi

	    echo "Seattle Firewall Installer Version $VERSION"
	    exit 0
	    ;;
	*)
	    if [ -n "$DEST" ]; then
		usage 1
            fi

	    DEST="$1"
	    ;;
    esac
    shift
    ARGS="yes"
done
    
#
# Determine where to install the firewall script
#
FIREWALL="firewall"

if [ -z "$DEST" ]; then
    if [ -d /etc/rc.d/init.d ]; then
	DEST=/etc/rc.d/init.d
    elif [ -d /etc/init.d ]; then
	DEST=/etc/init.d
    elif [ -f /etc/rc.d/rc.local ]; then
	DEST=/etc/rc.d
	FIREWALL="rc.firewall"
    else
	echo "ERROR: Can't determine where to install the firewall script"
	echo "       Rerun $0 passing the name of the SysVInit script directory"
	echo "       on your system"
	exit 1
    fi
fi

#
# Change to the directory containing this script
#
cd "`dirname $0`"

#
# Install the firewall script
#
echo "Installing Seattle Firewall Version $VERSION"

if [ -n "$RUNLEVELS" ]; then
    #
    # User specified chkconfig parameters -- build an awk script to install them
    # in the firewall script
    #
    echo "/# chkconfig/ { print \"# chkconfig: $RUNLEVELS\" ; next }" > awk.temp
    echo "{ print }" >> awk.temp

    awk -f awk.temp firewall > firewall.temp

    if [ $? -ne 0 ]; then
	echo -e "\nERROR: Error running awk."
	echo    "         You must run `basename $0` without the "-r" option then edit"
	echo    "         $DEST/$FIREWALL  manually (line beginning '# chkconfig:')"
	exit 1
    fi
    
    run_install -o root -g root -m 0544 firewall.temp $DEST/$FIREWALL

    rm -f firewall.temp awk.tmp
else
    run_install -o root -g root -m 0544 firewall $DEST/$FIREWALL
fi
    
echo -e "\nSeattle Firewall script installed in $DEST/$FIREWALL"
#
# Install the control program
#
run_install -o root -g root -m 0544 seawall /sbin

echo -e "\nSeattle Firewall control program installed in /sbin/seawall"
#
# Install seawall.conf
#
if [ -f /etc/seawall.conf ]; then
    run_install -o root -g root -m 0744 seawall.conf /etc/seawall.conf.new
    echo -e "\nNew configuration file placed in /etc/seawall.conf.new"
else
    run_install -o root -g root -m 0744 seawall.conf /etc
    echo -e "\nConfiguration file installed as /etc/seawall.conf"
    first_install="Yes"
fi
#
# Create /etc/seawall if needed
#
if [ ! -d /etc/seawall ]; then
    mkdir /etc/seawall
fi
#
# Install the apps file
#
if [ ! -f /etc/seawall/apps ]; then
    run_install -o root -g root -m 0744 apps /etc/seawall/apps
    echo -e "\nApps file installed as /etc/seawall/apps"
fi
#
# Install the servers file
#
if [ ! -f /etc/seawall/servers ]; then
    run_install -o root -g root -m 0744 servers /etc/seawall/servers
    echo -e "\nServers file installed as /etc/seawall/servers"
fi
#
# Create the version file
#
echo "$VERSION" > /etc/seawall/version
#
# Remove and create the symbolic link to the firewall script
#
rm -f /etc/seawall/firewall

ln -s "$DEST/$FIREWALL" /etc/seawall/firewall

echo -e "\n/etc/seawall/firewall linked to $DEST/$FIREWALL"

if [ -n "$first_install" ]; then
    CHKCONFIG="`which chkconfig`"
    if [ -n "$CHKCONFIG" -a -x "$CHKCONFIG"  ]; then
	if ( chkconfig --add $FIREWALL ); then
	    echo -e "\nFirewall will automatically start in run levels as follows:"
	    chkconfig --list $FIREWALL
	else
	    cant_autostart
	fi
    else
       modify_rclocal
    fi
fi
#
#  Report Success
#
echo -e "\nSeattle Firewall Version $VERSION Installed"
